//Crystal at 12.0MHz 1MIPS
#define IIC_SDA P3_2                          // 将p3.2口模拟数据口，必须要这样。中断接受数据
#ifdef IIC_SCL
#else
#define IIC_SCL P1_6                          // 将p1.1口模拟时钟口，默认
#endif
#ifdef SlaveAddress
#else
#define SlaveAddress 0x03		  //地址
#endif
#ifdef MasterAddress
#else
#define MasterAddress 0x01		  //主机地址
#endif

unsigned char s_control;
unsigned char s_note;
unsigned char s_svel;
bit	nm;
bit   SystemError;                // 从机错误标志位
//--------------------------------------------------------------------------------------------------
// 函数名称： iic_start()
// 函数功能： 启动IIC总线子程序
//--------------------------------------------------------------------------------------------------
void iic_start(void)
{ 	EA=0;            //时钟保持高，数据线从高到低一次跳变，IIC通信开始
	IIC_SDA = 1;         
	IIC_SCL = 1;
	delayNOP(5);      // 延时5us 
	IIC_SDA = 0;
	delayNOP(5);
	IIC_SCL = 0;
}
//--------------------------------------------------------------------------------------------------
// 函数名称： iic_stop()
// 函数功能： 停止IIC总线数据传送子程序
//--------------------------------------------------------------------------------------------------
void iic_stop(void)
{ 
	IIC_SDA = 0;   	   //时钟保持高，数据线从低到高一次跳变，IIC通信停止
	IIC_SCL = 1;
	delayNOP(5);
	IIC_SDA = 1;
	delayNOP(5);
	IIC_SCL = 0;
	EA=1;
}
//--------------------------------------------------------------------------------------------------
// 函数名称： slave_ACK
// 函数功能： 从机发送应答位子程序
//--------------------------------------------------------------------------------------------------
void slave_ACK(void)
{
	IIC_SDA = 0;    
	IIC_SCL = 1;
	delayNOP(5);			
	IIC_SDA = 1;
	IIC_SCL = 0;
}
//--------------------------------------------------------------------------------------------------
// 函数名称： slave_NOACK
// 函数功能： 从机发送非应答位子程序，迫使数据传输过程结束
//--------------------------------------------------------------------------------------------------
void slave_NOACK(void)
{ 
	IIC_SDA = 1;   
	IIC_SCL = 1;
	delayNOP(5);
	IIC_SDA = 0;
	IIC_SCL = 0;
}
//--------------------------------------------------------------------------------------------------
// 函数名称： check_ACK
// 函数功能： 主机应答位检查子程序，迫使数据传输过程结束
//--------------------------------------------------------------------------------------------------
void check_ACK(void)
{ 

	IIC_SDA = 1;      // 将p1.0设置成输入，必须先向端口写1
	IIC_SCL = 1;
	F0 = 0;
	if(IIC_SDA == 1)    // 若IIC_SDA=1表明非应答，置位非应答标志F0
		F0 = 1;
	IIC_SCL = 0;
}

//--------------------------------------------------------------------------------------------------
// 函数名称： IICSendByte
// 入口参数： ch
// 函数功能： 发送一个字节
//--------------------------------------------------------------------------------------------------
void IICSendByte(unsigned char ch)
{ 
	unsigned char idata n=8;     // 向IIC_SDA上发送一位数据字节，共八位
	while(n--)
	{ 
		if((ch&0x80) == 0x80)    // 若要发送的数据最高位为1则发送位1
		{
			IIC_SDA = 1;    // 传送位1
			IIC_SCL = 1;
			delayNOP(5);
			IIC_SDA = 0;
			IIC_SCL = 0;    
		}
		else
		{  
			IIC_SDA = 0;    // 否则传送位0
			IIC_SCL = 1;
			delayNOP(5);
			IIC_SCL = 0;  
		}
		ch = ch<<1;    // 数据左移一位
	}
}
//--------------------------------------------------------------------------------------------------
// 函数名称： IICreceiveByte
// 返回接收的数据
// 函数功能： 接收一字节子程序
//--------------------------------------------------------------------------------------------------
unsigned char IICreceiveByte(void)
{
	unsigned char idata n=8;    // 从IIC_SDA线上读取一上数据字节，共八位
	unsigned char tdata=0;
	while(n--)
	{
		IIC_SDA = 1;
		IIC_SCL = 1;
		tdata = tdata<<1;    // 左移一位，或_crol_(temp,1)
		if(IIC_SDA == 1)
			tdata = tdata|0x01;    // 若接收到的位为1，则数据的最后一位置1
		else 
			tdata = tdata&0xfe;    // 否则数据的最后一位置0
		IIC_SCL=0;
	}
	return(tdata);
}

bit IICwaitACK()
{ //10us不属于超时
unsigned char i=0;
bit j=0;
IIC_SDA=1;//输入状态
while (i!=15) //等15uS
{
	if ((IIC_SCL==1)&(IIC_SDA=0)) //先SCL=1 SDA=0后SCL=0 SDA=1 若SCL=1（第一次）
	{
		j=1; //OK,等到了ACK
		break; //走人
	}
	i++; //计数器+1
}

return j;

}
//--------------------------------------------------------------------------------------------------
// 函数名称： slavesenddata
// 入口参数： control,note and vel
// 函数功能： 发送MIDI信息在IIC总线上
//--------------------------------------------------------------------------------------------------
//保证Simple.不使用标准IIC协议。 发送从机地址，等ACK,再发送发送3byte数据，等ACK.It 's very simple.
void slavesenddata(unsigned char control,unsigned char note,unsigned char svel)
{
EA=0;
IT0=0; //外中断0为 下降沿触发 设定成低电平出发的话 容易导致 误触发
EX0=0; //开外部中断0
iic_start();
SystemError=1;
IICSendByte(MasterAddress);//发送地址
if (IICwaitACK()) //等主机的回应
{
IICSendByte(SlaveAddress); //发送本机地址
if (IICwaitACK()) //等主机的回应
{
IICSendByte(control);
if (IICwaitACK()){
IICSendByte(note);
if (IICwaitACK()){
IICSendByte(svel);
if (IICwaitACK()){
SystemError=0;
}
}
}
}
}
iic_stop();
IT0=1; //地址不对就不去管了
EX0=1; //开中断继续
EA=1;
}
void initial_iic()
{
IT0=1; //外中断0为 下降沿触发 设定成低电平出发的话 容易导致 误触发
EX0=1; //开外部中断0
EA=1;
}
void recvint0() interrupt 0 using 2
{
unsigned char mAddress=0;
EA=0;
IT0=0; //
EX0=0; //关中断防止干扰
//slave_ACK();
mAddress=IICreceiveByte();
if (mAddress==SlaveAddress)
	{
		slave_ACK();//是自己的地址就收下来
		if(IICreceiveByte()==MasterAddress)//判断是否是主控发送的
		{
		slave_ACK();
		s_control=IICreceiveByte(); //每次接受到一个数据就去ACK下
		slave_ACK();
		s_note=IICreceiveByte();
		slave_ACK();
		s_svel=IICreceiveByte();
		slave_ACK();
		nm=1; //提示主程序有新的消息，请注意查收～
		}
		else
		{
			slave_NOACK();//不是主机发送的就NAK
		}
	}
else
	{//延时，不能这样退出去引发中断
//ACK占用两个周期，接受数据占用8个周期。延够时了就跑,当然保险点还要Hang在这里2uS
		delayNOP(32);
	}
IT0=1; //地址不对就不去管了
EX0=1; //开中断继续
EA=1;
} 
